local Apollo = require "Apollo"
local GameLib = require "GameLib"
local GroupLib = require "GroupLib"

local NLLC = Apollo.GetAddon("NextLabLootClient")

local EnumColorType = {
	Rank = 1,
	Class = 2,
	UnitName = 3,
}
local SavedTypes = {
	EnumColorType.Rank,
	EnumColorType.Class,
}

local Colors = {
	EnumColorType = EnumColorType,
	tColors = {
		[EnumColorType.Rank] = {
			[7] = "ff8e7cc3",
			[6] = "ff6fa8dc",
			[5] = "ffa4c2f4",
			[4] = "ff93c47d",
			[3] = "ffffd966",
			[2] = "fff6b26b",
			[1] = "ffe06666",
		},
		[EnumColorType.Class] = {
			[GameLib.CodeEnumClass.Warrior] = "FFF54F4F",
			[GameLib.CodeEnumClass.Engineer] = "FFFFC830",
			[GameLib.CodeEnumClass.Esper] = "FF1591DB",
			[GameLib.CodeEnumClass.Medic] = "FF00D030",
			[GameLib.CodeEnumClass.Stalker] = "FFD23EF4",
			[GameLib.CodeEnumClass.Spellslinger] = "FFF47E2F",
		},
		[EnumColorType.UnitName] = {
		},
	},
	callbacks = {
		[EnumColorType.Rank] = {}, --.EnumColorType.Rank
		[EnumColorType.Class] = {},
		[EnumColorType.UnitName] = {},
	},
}
NLLC.Colors = Colors
Apollo.LinkAddon(NLLC, Colors)

--------------------------------
-- General Stuff (all Colors) --
--------------------------------
function Colors:OnDocLoaded()
	self:RegisterOnColorsChanged(EnumColorType.Class, "UpdateGroupClasses", self)
	Apollo.RegisterEventHandler("Group_Left", "UpdateGroupClasses", self)
	Apollo.RegisterEventHandler("Group_Join", "UpdateGroupClasses", self)
	Apollo.RegisterEventHandler("Group_Add", "UpdateGroupClasses", self)
	Apollo.RegisterEventHandler("Group_Remove", "UpdateGroupClasses", self)
	self:UpdateGroupClasses()
end
NLLC:OnceDocLoaded("OnDocLoaded", Colors)

function Colors:RegisterOnColorsChanged(eColorType, func, handler, ...)
	if not self.callbacks[eColorType] then error("ColorType not supported: "..tostring(eColorType)) end

	local callback = self:CreateCallback(func, handler, ...)
	table.insert(self.callbacks[eColorType], callback)

	return self:GetColors(eColorType)
end

function Colors:GetColors(eColorType)
	if not self.tColors[eColorType] then error("ColorType not supported: "..tostring(eColorType)) end

	return self.tColors[eColorType]
end

function Colors:PerformCallbacks(eColorType)
	if not eColorType then
		for _, eColor in pairs(EnumColorType) do
			self:PerformCallbacks(eColor)
		end
	else
		if not self.callbacks[eColorType] then error("ColorType not supported: "..tostring(eColorType)) end
		for i, callback in ipairs(self.callbacks[eColorType]) do
			callback(Colors:GetColors(eColorType), eColorType)
		end
	end
end

function Colors:CreateCallback(func, handler, ...)
	local t = {...}
	if type(func) == "function" then
		return function(arg1, arg2) return func(arg1, arg2, unpack(t)) end
	elseif type(handler) == "table" then
		return function(arg1, arg2) return handler[func](handler, arg1, arg2, unpack(t)) end
	else
		error("Could not create callback, need either a function, or a key-table pair.")
	end
end

function Colors:OnSave(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.General then
		tSavedData.Colors = {}
		for _, eColor in pairs(EnumColorType) do
			tSavedData.Colors[eColor] = self.tColors[eColor]
		end
	end
end
NLLC:OnceSaving("OnSave", Colors)

function Colors:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.General then
		if not tSavedData or not tSavedData.Colors then return end
		for _, eColor in pairs(SavedTypes) do
			self.tColors[eColor] = tSavedData.Colors[eColor] or self.tColors[eColor]
			self:PerformCallbacks(eColor)
		end
	end
end
NLLC:OnceRestored("OnRestore", Colors)

-------------------------------------------------------
-- Unitname Coloring (Class Colors for Groupmembers) --
-------------------------------------------------------

function Colors:UpdateGroupClasses()
	local tbl = {}
	self.tColors[EnumColorType.UnitName] = tbl
	local class = self.tColors[EnumColorType.Class]

	for i = 1, GroupLib.GetMemberCount(), 1 do
		local mbr = GroupLib.GetGroupMember(i)
		tbl[mbr.strCharacterName] = class[mbr.eClassId]
	end

	self:PerformCallbacks(EnumColorType.UnitName)
end
