local Apollo = require "Apollo"
local bit32 = require "bit32"

local NLLC = Apollo.GetAddon("NextLabLootClient")

local Condenser = {}
Condenser.__index = Condenser
NLLC.Condenser = Condenser

Apollo.LinkAddon(NLLC, Condenser)

--[[ General Thought: (at the cost of alot of assumptions: make shared tables much shorter)

un-condesed:
	{ ["width"] = 100,
	  ["height"] = 150,
	  ["text"] = "",
	  ["bgPicture"] = "",
	  ["font"] = "sadf"}

condensed:
	keys = {"width", "height", "text", "bgPicture", "font"}
	defaults = {
		["width"] = 75,
		["height"] = 75,
		["text"] = "",
		["bgPicture"] = "",
		["font"] = "",}
	--> We assume, that both keys AND defaults never change, and as such
		are known by every Client recieving this message!
		(it can be extended, but not changed with all backwards-compatability)
	--> Note: text and bgPicture match the default.

	{ 19, 100, 150, "sadf"} <- this is obviously MUCH shorter, but comes with ALOT of assumptions.
	--> 19 = 00...0010011 (meaning: first, scnd keys are given(1); thrid, fourth are default(0);
			fifth is given(1) and rest is default(0))
]]


-- caution: compare(nil, nil) = true!
local function compare(a, b) --returns true for equal.
	local t = type(a)
	if t == type(b) then
		if t == "table" then
			for i,v in ipairs(a) do
				if not compare(v, b[i]) then
					return false
				end
			end
			return true
		else
			return a == b
		end
	else
		return false
	end
end

local function copy(a)
	if type(a) == "table" then
		local c = {}
		for i,v in pairs(a) do
			c[copy(i)] = copy(v)
		end
		return c
	else
		return a
	end
end

local bit = bit32
local bits = {}
for i=1,32,1 do bits[i] = bit.lshift(1,i-1) end

--try to put frequently non-default values to the first keys, others further to the back.
function Condenser.new(keys, defaults)
	if #keys > 32 then
		error("Too many keys for one Condenser! ("..tostring(#keys).."; max: 32)")
	end

	local o = {
		kys = keys,
		def = defaults,
	}
	setmetatable(o, Condenser)
	return o
end

function Condenser:To(tbl, bPreventDefaults) --prevets comparison with defaults -> all matching keys will be converted.
	local ret = {0} --reserve the first spot.
	local num = 0

	if bPreventDefaults then
		for i, key in ipairs(self.kys) do
			if tbl[key] ~= nil then
				num = bit.bor(num, bits[i])
				ret[#ret+1] = tbl[key]
			end
		end
	else
		for i, key in ipairs(self.kys) do
			if not compare(self.def[key], tbl[key]) and tbl[key]~=nil then
				num = bit.bor(num, bits[i])
				ret[#ret+1] = tbl[key]
			end
		end
	end

	ret[1] = num

	return ret
end

function Condenser:From(tbl, bPreventDefaults) --prevents filling with defaults, if not given by tbl
	tbl = tbl or {}
	bPreventDefaults = not bPreventDefaults --dont wanna negate everytime... (yes, the name is wrong then.)
	local ret = {}
	local num = tbl[1] or 0

	local next = ipairs(tbl)
	local idx, val = 1;

	for i, key in ipairs(self.kys) do
		local b = bits[i]
		if bit.btest(num, b) then
			idx, val = next(tbl, idx)
			ret[key] = val
			if val == nil then
				error("Expected a value for: "..tostring(key)..", but got nothing")
			end
		elseif bPreventDefaults then--assume default
			ret[key] = copy(self.def[key])
		end
	end

	return ret
end

function Condenser:IsEqual(tbl1, tbl2)
	if tbl1[1] ~= tbl2[1] then return false end
	if #tbl1 ~= #tbl2 then return false end --should not happen... because above.

	for i = 2, #tbl1, 1 do
		if tbl1[i] ~= tbl2[i] then return false end
	end

	return true
end
