local Apollo = require "Apollo"
local GroupLib = require "GroupLib"

local NLLC = Apollo.GetAddon("NextLabLootClient")
local Log = NLLC.Log

local Lead = {
	loaded = false,
	leader = nil,
	bSelf = true, --whether you are the leader (ungrouped = true)
	callbacks = {},
}
NLLC.Lead = Lead
Apollo.LinkAddon(NLLC, Lead)

-----------------------
-- Acessible Methods --
-----------------------

function Lead:GetLeader()
	return self.leader, self.bSelf --this can be nil, if not loaded yet, or there currently is no leader.
end

function Lead:RegisterOnLeadChanged(func, handler, ...)
	local callback = self:CreateCallback(func, handler, ...)
	table.insert(self.callbacks, callback)

	if self.loaded then
		callback(self:GetLeader())
	end
end

-----------------------
-- Internal Stuff --
-----------------------

function Lead:CreateCallback(func, handler, ...)
	local t = {...}
	if type(func) == "function" then
		return function(arg1, arg2) return func(arg1, arg2, unpack(t)) end
	elseif type(handler) == "table" then
		return function(arg1, arg2) return handler[func](handler, arg1, arg2, unpack(t)) end
	else
		error("Could not create callback, need either a function, or a key-table pair.")
	end
end

function Lead:OnDocLoaded()
	self.loaded = true
	Apollo.RegisterEventHandler("Group_MemberPromoted", "OnLeaderChanged", self)
	Apollo.RegisterEventHandler("Group_Join", "CheckLeader", self)
	Apollo.RegisterEventHandler("Group_Left", "CheckLeader", self)

	self:CheckLeader()
end
NLLC:OnceDocLoaded("OnDocLoaded", Lead)

function Lead:PerformCallbacks()
	local strLead, bSelf = self:GetLeader()
	Log:Add("L", strLead, bSelf )
	for _, callback in ipairs(self.callbacks) do
		callback(strLead, bSelf)
	end
end

function Lead:OnLeaderChanged(strName, bSelf)
	self.leader = strName
	self.bSelf = bSelf
	self:PerformCallbacks()
end

function Lead:CheckLeader()
	if not GroupLib.InGroup() then
		self.leader = nil
		self.bSelf = true
		self:PerformCallbacks()
		return
	end

	for i = 1, GroupLib.GetMemberCount(), 1 do
		local tMember = GroupLib.GetGroupMember(i)
		if tMember.bIsLeader then
			self.leader = tMember.strCharacterName
			self.bSelf = i==1
			self:PerformCallbacks()
			return
		end
	end

	self.leader = nil
	self.bSelf = true
	self:PerformCallbacks()
end
