local Apollo = require "Apollo"

local NLLC = Apollo.GetAddon("NextLabLootClient")
local CHANNEL_NAME = "ChnlNextLabLoot"

local Log = {}
NLLC.Log = Log

local preloadStack = {}
local SendVarToRover = function() end; --this will get replaced by the actual, when manually asked to debug.

Apollo.LinkAddon(NLLC, Log)

function Log:Add(...)
	local d = {...}
	if not self.loaded then
		table.insert(preloadStack, d)
	else
		self.vHandler:Save(nil, d)
	end
end

function Log:OnLoad()
	Apollo.RegisterSlashCommand("nllcLog", "OnSlashCommand", self)
	self.vHandler = NLLC.VersionHandler.new(CHANNEL_NAME, nil, "D")
	self.vHandler:SetLogging(false) --disable logging the very logging-data.
	self.loaded = true
	for i,d in ipairs(preloadStack) do
		self.vHandler:Save(nil, d)
	end
	preloadStack = nil
end
NLLC:OnceLoaded("OnLoad", Log)

function Log:OnSlashCommand(cmd, args)
	if not _G.SendVarToRover then return end
	SendVarToRover = _G.SendVarToRover
	local name;
	if args and type(args) == "string" then
		name = args:match("^%S+%s%S+")
	end

	if not name then Print("Abort! No Name!") return end

	self.strNameLogged = name
	self.bAwaitingVersion = true

	self.vHandler:OnAdded("OnDataAdded", self)
	self.vHandler:OnNewVersion("OnNewVersion", self)
	SendVarToRover("RequestedDebug | "..name, name)
	self.vHandler:RequestData(name, {1}, name, false) --a version, or structure request would result in nothing!
	--requesting (existing) data, tho will end with us recieving anything he knows.
end

function Log:OnDataAdded(prefix, type, owner, key, data)
	if owner ~= self.strNameLogged then return end
	local maxIdx = 0
	for i,v in pairs(data) do
		if i>maxIdx then maxIdx=i end
	end
	local tbl = {}
	for i=1,maxIdx,1 do
		tbl[i] = tostring(data[i])
	end
	SendVarToRover(owner.." | "..key, table.concat(tbl, " | "))
end

local function fullTbl(from, to)
	local tbl, offset = {}, 1-from
	for i = from, to, 1 do
		tbl[i+offset] = i
	end
	return tbl
end

function Log:OnNewVersion(prefix, owner, tVersions)
	if self.bAwaitingVersion and owner == self.strNameLogged then
		self.bAwaitingVersion = false
		SendVarToRover("AquiredVersion | "..tostring(tVersions.major), tVersions.major)
		local idTbl = fullTbl(2, tVersions.major)
		self.vHandler:RequestData(owner, idTbl, owner, false)
	end
end
