local Apollo = require "Apollo"
local GameLib = require "GameLib"
require "Item"

local NLLC = Apollo.GetAddon("NextLabLootClient")
local CHANNEL_NAME = "ChnlNextLabLoot"

local LootShare = {
	loaded = false,
	vHandler = nil,
	tItems = {},
	tLinks = {},
	callbacks = {},
}
NLLC.LootShare = LootShare
Apollo.LinkAddon(NLLC, LootShare)

local Lead = NLLC.Lead
local VersionHandler = NLLC.VersionHandler
local Condenser = NLLC.Condenser.new({
	"strReciever",
	"strItemLink",
	"nItemId",
	"strItemNote",
},{
	["strReciever"] = "",
	["strItemLink"] = "",
	["nItemId"] = 0,
	["strItemNote"] = "M", --other values are (for example) 'O5', 'M10'
})

-----------------------
-- Acessible Methods --
-----------------------

function LootShare:Clear()
	if not self.loaded then error("LootShare not initialized yet") end

	self.vHandler:RemoveAll()
end

function LootShare:AddLoot(tItem, strReciever, strNote)
	if not self.loaded then error("LootShare not initialized yet") end
	local tData = Condenser:To({
		strReciever = strReciever,
		strItemNote = strNote,
		nItemId = tItem:GetItemId(),
		strItemLink = tItem:GetChatLinkString(),
	})

	self.vHandler:Save(nil, tData)
end

function LootShare:RegisterOnLootChanged(func, handler, ...)
	local callback = self:CreateCallback(func, handler, ...)
	table.insert(self.callbacks, callback)

	if self.loaded then
		callback(self.tRanks)
	end
end

-----------------------
-- Internal Stuff --
-----------------------

function LootShare:CreateCallback(func, handler, ...)
	local t = {...}
	if type(func) == "function" then
		return function(arg) return func(arg, unpack(t)) end
	elseif type(handler) == "table" then
		return function(arg) return handler[func](handler, arg, unpack(t)) end
	else
		error("Could not create callback, need either a function, or a key-table pair.")
	end
end

function LootShare:OnSave(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.General then
		tSavedData.LootShare = self.vHandler and self.vHandler:GetData()
	end
end
NLLC:OnceSaving("OnSave", LootShare)

function LootShare:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.General then
		local data = tSavedData and tSavedData.LootShare or nil
		self.vHandler = VersionHandler.new(CHANNEL_NAME, data, "L")
		self.vHandler:OnAdded("VHLootAdded", self)
		self.vHandler:OnRemoved("VHLootRemoved", self)
		self.vHandler:OnChanged("VHLootChanged", self)

		Lead:RegisterOnLeadChanged("LeaderChanged", self)
	end
end
NLLC:OnceRestored("OnRestore", LootShare)


function LootShare:PerformCallbacks(arg)
	for _, callback in ipairs(self.callbacks) do
		callback(arg)
	end
end

function LootShare:LeaderChanged(strLead, bIsSelf)
	self.loaded = true
	self.tItems = {}
	self.tLinks = {}
	if strLead ~= nil and self.vHandler:HasOwner(strLead) then
		local i = 1
		for key, data in self.vHandler:Iterate() do
				local tData = Condenser:From(data)
				self.tItems[i] = tData
				self.tLinks[tData.strItemLink] = i
				i = i+1
		end
	end
	self:PerformCallbacks(self.tItems)

	self.vHandler:SetSharing(bIsSelf)
end

function LootShare:VHLootAdded(prefix, type, owner, key, data)
	if owner ~= Lead:GetLeader() then return end

	local tData = Condenser:From(data)

	local i = #self.tItems+1
	self.tItems[i] = tData
	self.tLinks[tData.strItemLink] = i

	self:PerformCallbacks(self.tItems)
end

function LootShare:VHLootRemoved(prefix, type, owner, key, data)
	if owner ~= Lead:GetLeader() then return end

	local tData = Condenser:From(data)

	local i = self.tLinks[tData.strItemLink]
	if not i then return end

	table.remove(self.tItems, i)

	--just redo it completely. then atleast we are sure its correct..
	local newLinks = {}

	for idx, dat in ipairs(self.tItems) do
		newLinks[dat.strItemLink] = idx
	end

	self.tLinks = newLinks

	self:PerformCallbacks(self.tItems)
end

function LootShare:VHLootChanged(prefix, type, owner, key, data)
	if owner ~= Lead:GetLeader() then return end

	local tData = Condenser:From(data)

	local i = self.tLinks[tData.strItemLink]
	if not i then return end

	self.tItems[i] = tData

	self:PerformCallbacks(self.tItems)
end
