--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Raid Looter
-- The MIT License (MIT)
--
-- Copyright (c) 2015 - 2016
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.
--
-- Debugging
-- /rl dev-mode [on|off]
--
-- Events
-- Incoming events passes throw the 'Utils:InvokeEventHandler' for debug purposes.
-- Sometimes events are forwarded directly by using 'Utils:FireEvent'
--
-- EXTRA NOTE
-- Since all Wildstar addons share the same global space it is important not to;
--     * relay on functions to exist,
--     * Redefine/change or remove standard functions.
--
-- ORIGINAL AUTHOR
-- Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar)
-- MODIFIED BY
-- Mischh X (Jabbit-EU / Wildstar)
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--

local Apollo = require "Apollo"

local NLLC = Apollo.GetAddon("NextLabLootClient")
NLLC.G = NLLC.G or {}
local G = NLLC.G

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Autoloot Dropdown Item
-- Represents an item in the dropdown.
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
local AutolootDropdownItem = {}
G.AutolootDropdownItem = AutolootDropdownItem
function AutolootDropdownItem:New(tRaidLooter, wndParent, strName)
	local o = {
		wndParent = wndParent,
		wndMain = nil,
		wndText = nil,
		tLinker = G.Linker:New(),
		bIsSelected = false,
		strName = strName,
		tRaidLooter = tRaidLooter,
		tCallback = nil,
	}

	setmetatable(o, self)
	self.__index = self
	return o
end

function AutolootDropdownItem:Init(tTarget)
	self.tLinker:Link(NLLC, self)
	self.wndMain = self.tRaidLooter:CreateWindow("Autoloot_DropdownItem", self.wndParent, self)
	self.wndText = self.wndMain:FindChild("BtnText")
	self.wndText:SetText(self.strName)
end

function AutolootDropdownItem:Destroy()
	self.wndMain:Destroy()
	self.wndMain     = nil
	self.wndParent   = nil
	self.tCallback   = nil
	self.wndParent   = nil
	self.tRaidLooter = nil
end

function AutolootDropdownItem:GetName() return self.strName end
function AutolootDropdownItem:IsSelected() return self.bIsSelected end
function AutolootDropdownItem:SetCallback(tCallback) self.tCallback = tCallback end

function AutolootDropdownItem:OnItemSelectionChanged(wndHandler, wndControl, eMouseButton)
	self.bIsSelected = wndControl:IsChecked()
	G.Utils:FireEvent(self.tCallback, "OnItemSelectionChanged", self)
end

function AutolootDropdownItem:OnCreateNewProfile(...)
	G.Utils:InvokeEventHandler(self, "AutolootDropdownItem", "OnItemSelectionChanged", ...)
end
