--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Raid Looter
-- The MIT License (MIT)
--
-- Copyright (c) 2015 - 2016
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.
--
-- Debugging
-- /rl dev-mode [on|off]
--
-- Events
-- Incoming events passes throw the 'Utils:InvokeEventHandler' for debug purposes.
-- Sometimes events are forwarded directly by using 'Utils:FireEvent'
--
-- EXTRA NOTE
-- Since all Wildstar addons share the same global space it is important not to;
--     * relay on functions to exist,
--     * Redefine/change or remove standard functions.
--
-- ORIGINAL AUTHOR
-- Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar)
-- MODIFIED BY
-- Mischh X (Jabbit-EU / Wildstar)
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--

local Apollo = require "Apollo"

local NLLC = Apollo.GetAddon("NextLabLootClient")
NLLC.G = NLLC.G or {}
local G = NLLC.G

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Autoloot Popup
-- Represents the autoloot view of the addon.
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
local AutolootPopup = {}
G.AutolootPopup = AutolootPopup
function AutolootPopup:New(tRaidLooter)
	local o = {
		wndRecipientEditBox = nil,
		wndLootEditBox      = nil,
		btnPlayer           = nil,
		btnLeader           = nil,
		btnRandom           = nil,
		btnCreateRule       = nil,
		tCallback           = nil,
		strPreviousText     = nil,
		tLinker             = G.Linker:New(),
		tRaidLooter         = tRaidLooter
	}

	setmetatable(o, self)
	self.__index = self
	return o
end

function AutolootPopup:Init(tTarget)
	self.tLinker:Link(NLLC, self)
	self.wndMain = self.tRaidLooter:CreateTopLevelWindow("Autoloot_RuleDropdown", self)
	self.wndRecipientEditBox = self.wndMain:FindChild("RecipientEditBox")
	self.wndLootEditBox = self.wndMain:FindChild("LootEditBox")
	self.wndCommentEditBox = self.wndMain:FindChild("CommentEditBox")
	self.btnPlayer = self.wndMain:FindChild("BtnPlayer")
	self.btnLeader = self.wndMain:FindChild("BtnLeader")
	self.btnRandom = self.wndMain:FindChild("BtnRandom")
	self.btnCreateRule = self.wndMain:FindChild("BtnCreateRule")

	G.Utils:Log(self.btnCreateRule)
	self.btnCreateRule:Enable(false)
	self.btnPlayer:SetCheck(true)
	self.wndMain:Show(false, true)
end

function AutolootPopup:Destroy()
	self.tLinker:Destroy()
	self.wndMain:Destroy()

	self.tLinker = nil
	self.wndMain = nil
	self.tRaidLooter = nil
	self.tCallback = nil
end

----------------------------------------------------------------------------------
-- Functions
----------------------------------------------------------------------------------
function AutolootPopup:GetWindow()
	return self.wndMain
end

-- Should Respond to:
--     OnNewAutolootRuleAdded(tRule)
--     OnNewAutolootRuleCancelled()
function AutolootPopup:SetCallback(tCallback)
	self.tCallback = tCallback
end

function AutolootPopup:Open(tPosition)
	self.wndMain:Invoke()
	self.wndMain:Move(tPosition.x, tPosition.y, self.wndMain:GetWidth(), self.wndMain:GetHeight())

	self.btnPlayer:SetCheck(true)
	self.btnLeader:SetCheck(false)
	self.btnRandom:SetCheck(false)
end

function AutolootPopup:Close()
	self.wndMain:Close()
end

function AutolootPopup:RevalidateCanCreate()
	local strRecipient = self.wndRecipientEditBox:GetText()
	local strItemName = self.wndLootEditBox:GetText()

	if strRecipient ~= nil and strItemName ~= nil then
		if strRecipient ~= "Leader" and strRecipient ~= "Randomize" then
			local strFirstname, strLastname = G.Utils:Split(strRecipient, " ")
			local bIsFirstnameValid = strFirstname ~= nil and strFirstname:len() >= 3
			local bIsLastnameValid  = strLastname ~= nil  and strLastname:len()  >= 3

			if not bIsFirstnameValid or not bIsLastnameValid then
				self.btnCreateRule:Enable(false)
				return
			end
		end

	if strItemName:len() >= 5 then
			self.btnCreateRule:Enable(true)
			return
		end
	end

	self.btnCreateRule:Enable(false)
end

----------------------------------------------------------------------------------
-- Event handlers
----------------------------------------------------------------------------------
function AutolootPopup:HandleOnAutolootPopupAdd()
	local strComment = self.wndCommentEditBox:GetText()
	local tRule = {
		strUnitName = self.wndRecipientEditBox:GetText(),
		strItemName = self.wndLootEditBox:GetText(),
		strComment  = strComment == nil and "" or strComment
	}

	G.Utils:FireEvent(self.tCallback, "OnNewAutolootRuleAdded", tRule)
end

function AutolootPopup:HandleOnRecipientTypeChanged(wndHandler, wndControl, eMouseButton)
	if wndControl:IsChecked() then
		if wndControl == self.btnPlayer then
			self.wndRecipientEditBox:SetText("")
			self.wndRecipientEditBox:SetTextColor("white")
			self.wndRecipientEditBox:Enable(true)
		elseif wndControl == self.btnLeader then
			self.wndRecipientEditBox:SetText("Leader")
			self.wndRecipientEditBox:SetTextColor("gray")
			self.wndRecipientEditBox:Enable(false)
		elseif wndControl == self.btnRandom then
			self.wndRecipientEditBox:SetText("Randomize")
			self.wndRecipientEditBox:SetTextColor("gray")
			self.wndRecipientEditBox:Enable(false)
		end
		self:RevalidateCanCreate()
	end
end

function AutolootPopup:OnRequiredEditboxChanged(wndHandler, wndControl, eMouseButton)
	self:RevalidateCanCreate()
end

function AutolootPopup:HandleOnAutoComplete(tSuggestions)
	if tSuggestions.strRecipient ~= nil then
		self.wndRecipientEditBox:SetText(tSuggestions.strRecipient)
	else
		self.wndRecipientEditBox:SetText("")
	end

	if tSuggestions.strItemName ~= nil then
		self.wndLootEditBox:SetText(tSuggestions.strItemName)
	end

	if tSuggestions.strComment ~= nil then
		self.wndCommentEditBox:SetText(tSuggestions.strComment)
	end

	self:RevalidateCanCreate()
end

----------------------------------------------------------------------------------
-- Events
----------------------------------------------------------------------------------
function AutolootPopup:OnAutolootPopupAdd(...)
	G.Utils:InvokeEventHandler(self, "AutolootPopup", "OnAutolootPopupAdd", ...) end
function AutolootPopup:OnRecipientTypeChanged(...)
	G.Utils:InvokeEventHandler(self, "AutolootPopup", "OnRecipientTypeChanged", ...) end
function AutolootPopup:OnAutoComplete(...) G.Utils:InvokeEventHandler(self, "AutolootPopup", "OnAutoComplete", ...) end
