--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Raid Looter
-- The MIT License (MIT)
--
-- Copyright (c) 2015 - 2016
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.
--
-- Debugging
-- /rl dev-mode [on|off]
--
-- Events
-- Incoming events passes throw the 'Utils:InvokeEventHandler' for debug purposes.
-- Sometimes events are forwarded directly by using 'Utils:FireEvent'
--
-- EXTRA NOTE
-- Since all Wildstar addons share the same global space it is important not to;
--     * relay on functions to exist,
--     * Redefine/change or remove standard functions.
--
-- ORIGINAL AUTHOR
-- Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar)
-- MODIFIED BY
-- Mischh X (Jabbit-EU / Wildstar)
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--

local Apollo = require "Apollo"

local NLLC = Apollo.GetAddon("NextLabLootClient")
NLLC.G = NLLC.G or {}
local G = NLLC.G

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Autoloot Row
-- Represents a row in the autoloot view.
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
local AutolootRule = {}
G.AutolootRule = AutolootRule
function AutolootRule:New(tRaidLooter, wndParent, strUnitName, strItemName)
	local o = {
		wndParent = wndParent,
		strUnitName = strUnitName,
		strItemName = strItemName,
		wndMain = nil,
		wndColumn1 = nil,
		wndColumn2 = nil,
		tCallback = nil,
		bShouldHighlight = false,
		nIndex = 0,
		tRaidLooter = tRaidLooter,
		tLinker = G.Linker:New()
	}

	setmetatable(o, self)
	self.__index = self
	return o
end

function AutolootRule:Init(tTarget)
	self.tLinker:Link(NLLC, self)

	self.wndMain = self.tRaidLooter:CreateWindow("Autoloot_Row", self.wndParent, self)
	self.wndColumn1 = self.wndMain:FindChild("Column1")
	self.wndColumn2 = self.wndMain:FindChild("Column2")
	self.wndHighlight = self.wndMain:FindChild("Highlight")
	self.wndMain:Show(true)
	self.wndColumn1:SetText(self.strUnitName)
	self.wndColumn2:SetText(self.strItemName)
	self.wndHighlight:Show(self.bShouldHighlight, true)
end

function AutolootRule:Destroy()
	self.tLinker:Destroy()
	if self.wndMain ~= nil then
		self.wndMain:Destroy()
	end

	self.wndMain     = nil
	self.tLinker     = nil
	self.tRaidLooter = nil
	self.tCallback   = nil
	self.wndParent   = nil
end

----------------------------------------------------------------------------------
-- Functions
----------------------------------------------------------------------------------
function AutolootRule:GetWindow()
	return self.wndMain
end

-- Should Respond to:
--     OnAutolootRowRowDeleted()
function AutolootRule:SetCallback(tCallback)
	self.tCallback = tCallback
end

function AutolootRule:SetIndex(nIndex)
	self.nIndex = nIndex
end

function AutolootRule:SetAnchor(tAnchor)
	local tP = tAnchor.tPoints
	local tO = tAnchor.tOffsets

	self.wndMain:SetAnchorPoints(tP[1], tP[2], tP[3], tP[4])
	self.wndMain:SetAnchorOffsets(tO[1], tO[2], tO[3], tO[4])
end

function AutolootRule:SetHighlight(bShouldHighlight)
	self.bShouldHighlight = bShouldHighlight
	self.wndHighlight:Show(self.bShouldHighlight, true)
end

function AutolootRule:GetIndex()
	return self.nIndex
end

function AutolootRule:GetRecipient()
	return self.wndColumn1:GetText()
end

function AutolootRule:GetLootName()
	return self.wndColumn2:GetText()
end

----------------------------------------------------------------------------------
-- Event handlers
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Events
----------------------------------------------------------------------------------
function AutolootRule:OnAutolootRowDeleted() G.Utils:FireEvent(self.tCallback, "OnAutolootRowDeleted", self) end
