--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Raid Looter
-- The MIT License (MIT)
--
-- Copyright (c) 2015 - 2016
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.
--
-- Debugging
-- /rl dev-mode [on|off]
--
-- Events
-- Incoming events passes throw the 'Utils:InvokeEventHandler' for debug purposes.
-- Sometimes events are forwarded directly by using 'Utils:FireEvent'
--
-- EXTRA NOTE
-- Since all Wildstar addons share the same global space it is important not to;
--     * relay on functions to exist,
--     * Redefine/change or remove standard functions.
--
-- ORIGINAL AUTHOR
-- Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar)
-- MODIFIED BY
-- Mischh X (Jabbit-EU / Wildstar)
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--

local Apollo = require "Apollo"

local NLLC = Apollo.GetAddon("NextLabLootClient")
NLLC.G = NLLC.G or {}
local G = NLLC.G

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Locale
-- Used to handle localization.
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
local Locale = {}
G.Locale = Locale
function Locale:Init(tTarget)
	self.tLinker = G.Linker:New()
	self.tLinker:Link(NLLC, self)
	self.strLanguage = self:DetermineCurrentLanguage()
end

function Locale:Destroy()
	self.tLinker:Destroy()
	self.tLinker = nil
end

----------------------------------------------------------------------------------
-- Functions
----------------------------------------------------------------------------------
function Locale:Translate(strKey)
	local strTranslation = self.tRegister[self.strLanguage][strKey]
	if strTranslation then
		return strTranslation
	else
		return strKey:upper().."KEY_MISSING"
	end
end

function Locale:TranslateCommand(strKey)
	local strTranslation = self.tRegister[self.strLanguage].Commands[strKey]
	if strTranslation then
		return strTranslation
	else
		return strKey:upper().."KEY_MISSING"
	end
end

function Locale:RegisterTranslations(strLangKey, tTranslations)
	self.tRegister = self.tRegister or {}
	self.tRegister[strLangKey] = tTranslations
end

----------------------------------------------------------------------------------
-- Private functions
----------------------------------------------------------------------------------
function Locale:DetermineCurrentLanguage()
	local strCancel = Apollo.GetString(1)
	if strCancel == "Abbrechen" then
		return "deDE"
	elseif strCancel == "Annuler" then
		return "frFR"
	else
		return "enUS"
	end
end

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- English Locale - enUS
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
Locale:RegisterTranslations("enUS", {
	Rolls = "rolls",
	Light = "Light",
	Medium = "Medium",
	Heavy = "Heavy",
	Pass = "Pass",
	Commands = {
		Roll = "/roll"
	}
})

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- French Locale - frFR
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
Locale:RegisterTranslations("frFR", {
	Rolls = "lance",
	Light = "Light",
	Medium = "Medium",
	Heavy = "Heavy",
	Pass = "Pass",
	Commands = {
		Roll = "/lancer"
	}
})

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- German Locale - deDE [Not supported yet.]
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
Locale:RegisterTranslations("deDE", {
	Rolls = "rolls",
	Light = "Leicht",
	Medium = "Mittelschwer",
	Heavy = "Schwer",
	Pass = "Passen",
	Commands = {
		Roll = "/würfeln"
	}
})
