--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Raid Looter
-- The MIT License (MIT)
--
-- Copyright (c) 2015 - 2016
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.
--
-- Debugging
-- /rl dev-mode [on|off]
--
-- Events
-- Incoming events passes throw the 'Utils:InvokeEventHandler' for debug purposes.
-- Sometimes events are forwarded directly by using 'Utils:FireEvent'
--
-- EXTRA NOTE
-- Since all Wildstar addons share the same global space it is important not to;
--     * relay on functions to exist,
--     * Redefine/change or remove standard functions.
--
-- ORIGINAL AUTHOR
-- Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar)
-- MODIFIED BY
-- Mischh X (Jabbit-EU / Wildstar)
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--

local Apollo = require "Apollo"

local NLLC = Apollo.GetAddon("NextLabLootClient")
NLLC.G = NLLC.G or {}
local G = NLLC.G

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- LootDetailsWindow
-- Represents the the details window for a specific loot item.
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
local LootDetailsWindow = {}
G.LootDetailsWindow = LootDetailsWindow
function LootDetailsWindow:New(tRaidLooter, tLootItem)
	local o = {
		wndMain = nil,
		wndName = nil,
		wndType = nil,
		wndDrop = nil,
		wndStack = nil,
		wndQueued = nil,
		tLootItem = tLootItem,
		tLinker = G.Linker:New(),
		tRaidLooter = tRaidLooter
	}

	setmetatable(o, self)
	self.__index = self
	return o
end

function LootDetailsWindow:Init(tTarget)
	self.tLinker:Link(NLLC, self)

	self.wndMain = self.tRaidLooter:CreateWindow("LootItem_Details", self)
	self.wndName = self.wndMain:FindChild("ItemName")
	self.wndType = self.wndMain:FindChild("ItemType")
	self.wndDrop = self.wndMain:FindChild("Dropped")
	self.wndStack = self.wndMain:FindChild("Stack")
	self.wndQueue = self.wndMain:FindChild("Queued")

	local strTextColor = G.Utils:GetItemQualityString(self.tLootItem)
	local nStackSize = self.tLootItem:GetStackSize()
	if nStackSize > 1 then
		self.wndStack:SetText(tostring(nStackSize).." pieces")
	else
		self.wndStack:SetText(tostring(nStackSize).." piece")
	end

	self.wndName:SetText(self.tLootItem:GetName())
	self.wndName:SetTextColor(strTextColor)
	self.wndType:SetText(self.tLootItem:GetTypeName())
	self.wndDrop:SetText(G.Utils:FormatDataToReadableApproximateTime(self.tLootItem:GetDropTime()))

	local strUnitName = self.tLootItem:GetQueueInfo()
	if strUnitName ~= nil then
		self.wndQueue:SetText("Yes, for "..strUnitName)
	else
		self.wndQueue:SetText("No")
	end
end

function LootDetailsWindow:Destroy()
	self.tLinker:Destroy()
	self.wndMain:Destroy()

	self.wndMain     = nil
	self.tLootItem   = nil
	self.tRaidLooter = nil
end

----------------------------------------------------------------------------------
-- Functions
----------------------------------------------------------------------------------
function LootDetailsWindow:Open(tPosition)
	local knXCursorOffset = 25
	local knYCursorOffset = 25

	tPosition.x = tPosition.x - knXCursorOffset
	tPosition.y = tPosition.y - knYCursorOffset

	self.wndMain:Invoke()
	self.wndMain:Move(tPosition.x, tPosition.y, self.wndMain:GetWidth(), self.wndMain:GetHeight())
end

function LootDetailsWindow:Close()
	self.wndMain:Close()
end
