--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Raid Looter
-- The MIT License (MIT)
--
-- Copyright (c) 2015 - 2016
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.
--
-- Debugging
-- /rl dev-mode [on|off]
--
-- Events
-- Incoming events passes throw the 'Utils:InvokeEventHandler' for debug purposes.
-- Sometimes events are forwarded directly by using 'Utils:FireEvent'
--
-- EXTRA NOTE
-- Since all Wildstar addons share the same global space it is important not to;
--     * relay on functions to exist,
--     * Redefine/change or remove standard functions.
--
-- ORIGINAL AUTHOR
-- Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar)
-- MODIFIED BY
-- Mischh X (Jabbit-EU / Wildstar)
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--

local Apollo = require "Apollo"

local NLLC = Apollo.GetAddon("NextLabLootClient")
NLLC.G = NLLC.G or {}
local G = NLLC.G

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- LootQueue
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
local LootQueue = { }
G.LootQueue = LootQueue
function LootQueue:New()
    local o = {
        tItemQueues = {},
        tLinker = G.Linker:New(),
        tMasterLoot = nil
    }

    setmetatable(o, self)
    self.__index = self
    return o
end

function LootQueue:Init(tTarget) self.tLinker:Link(NLLC, self) end
function LootQueue:Destroy()
    self.tLinker:Destroy()
    self.tItemQueues = nil
    self.tMasterLoot = nil
end

----------------------------------------------------------------------------------
-- Functions
----------------------------------------------------------------------------------
function LootQueue:SetMasterLoot(tMasterLoot)
    self.tMasterLoot = tMasterLoot
    self.tMasterLoot:RegisterCallback(self)
end

function LootQueue:Clear()
    self.tItemQueues = {}
end

function LootQueue:Push(tLootItem, strUnitName)
    G.Utils:Log("Pushing "..tLootItem:GetName().." to "..strUnitName.." QUEUE")
    if tLootItem == nil or strUnitName == nil then
        return
    end

    local tItemQueue = self.tItemQueues[strUnitName]
    if tItemQueue == nil then
        tItemQueue = {}
        self.tItemQueues[strUnitName] = tItemQueue
    end

    for nIndex = 1, #tItemQueue do
        if tItemQueue[nIndex]:HaveId(tLootItem:GetId()) then
            G.Utils:Log("ITEM - Already in the queue.")
            return -- Already in the queue.
        end
    end

    tLootItem:SetIsQueued(true, strUnitName)
    table.insert(tItemQueue, tLootItem)
end

function LootQueue:Remove(tLootItem)
    tLootItem:SetIsQueued(false)
    local nLootId = tLootItem:GetId()
    for strUnitName, tItemQueue in pairs(self.tItemQueues) do
        for nIndex, tExistingLootItem in ipairs(tItemQueue) do
            if tExistingLootItem:GetId() == nLootId then
                table.remove(tItemQueue, nIndex)
                if #tItemQueue == 0 then
                    self.tItemQueues[strUnitName] = nil
                end
                break
            end
        end
    end
end

----------------------------------------------------------------------------------
-- Event handlers
----------------------------------------------------------------------------------
function LootQueue:HandleOnLootUpdate(tLoot)
    G.Utils:Log(tLoot)
    if G.Utils:TableLength(self.tItemQueues) > 0 and #tLoot > 0 then
        for _, tUnit in pairs(tLoot[1].tLooters) do
            local strUnitName = tUnit:GetName()
            local tItemQueue = self.tItemQueues[strUnitName]

            G.Utils:Log("Unit \""..strUnitName.."\", itemQueue size: "..(tItemQueue and #tItemQueue or 0))

            if tItemQueue ~= nil then
                self.tMasterLoot:Assign(tItemQueue, {
                    tUnit = tUnit,
                    strUnitName = strUnitName
                })
                -- tItemQueue = nil
            end
        end
    else
        G.Utils:Log("No items queued")
    end
end

----------------------------------------------------------------------------------
-- Events
----------------------------------------------------------------------------------
function LootQueue:OnLootUpdate(...) G.Utils:InvokeEventHandler(self, "LootQueue", "OnLootUpdate", ...) end
