--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Raid Looter
-- The MIT License (MIT)
--
-- Copyright (c) 2015 - 2016
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.
--
-- Debugging
-- /rl dev-mode [on|off]
--
-- Events
-- Incoming events passes throw the 'Utils:InvokeEventHandler' for debug purposes.
-- Sometimes events are forwarded directly by using 'Utils:FireEvent'
--
-- EXTRA NOTE
-- Since all Wildstar addons share the same global space it is important not to;
--     * relay on functions to exist,
--     * Redefine/change or remove standard functions.
--
-- ORIGINAL AUTHOR
-- Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar)
-- MODIFIED BY
-- Mischh X (Jabbit-EU / Wildstar)
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--

local Apollo = require "Apollo"
local GameLib = require "GameLib"

local NLLC = Apollo.GetAddon("NextLabLootClient")
NLLC.G = NLLC.G or {}
local G = NLLC.G

--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
-- Settings
-- Abstracation for reading/writing settings.
--
-- Put all defaults in the defaults object in the constructor.
--::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::--
local Settings = {}
G.Settings = Settings
function Settings:New()
	local o = {
		bIsLoaded = false,
		tDefaults = {
			strVersion = G.CURRENT_VERSION,
			tGeneral = {
				bUseRollPopups = true,
				bOpenOnLoot    = true,
				bUseOverlay    = true,
				bUseSound      = false,
				bUseRollSound  = false,
				bUseLootSound  = false,
			},
			tLeader  = {
				bUseAutoSelection    = true,
				bAllowMultiSelection = true,
				bDetailedInspect     = false,
				bAutolootEnabled     = false,
				bAscending           = false
			},
			tDebug = {
				bIsTest                = false,
				bIsTracingEvents       = false,
				bIsDebug               = false,
				bShouldLink            = false,
				bIsLootSourceInventory = false,
				bIsLeader              = false,
				bIsAssistant           = false,
				bIsInRaid              = false,
			},
			tData = {
				tWindowLocations = {
					wndOverlayLoc = {},
				    wndLootWindowLoc = {},
					wndManageWindowLoc = {},
				},
				tAutolootProfiles = {
					["Default - Raid Commodity"] = {
						bIsDefault = true,
						bIsLoadable = false,
						nCreatedAt = 0,
						strDisplayName = "N/A",
						tLootList = {
							["Divine Class Focus - Major"] = { strRecipient = "Leader",    strComment = "Runecrafting" },
							["Divine Class Focus - Minor"] = { strRecipient = "Leader",    strComment = "Runecrafting" },
							["Divine Set Focus - Major"]   = { strRecipient = "Leader",    strComment = "Runecrafting" },
							["Divine Set Focus - Minor"]   = { strRecipient = "Randomize", strComment = "Runecrafting" },
							["Pure Class Focus - Major"]   = { strRecipient = "Leader",    strComment = "Runecrafting" },
							["Pure Class Focus - Minor"]   = { strRecipient = "Leader",    strComment = "Runecrafting" },
							["Pure Set Focus - Major"]     = { strRecipient = "Leader",    strComment = "Runecrafting" },
							["Pure Set Focus - Minor"]     = { strRecipient = "Randomize", strComment = "Runecrafting" },
							["Datascape Matrix"]           = { strRecipient = "Leader",    strComment = "Crafting" },
							["Genetic Matrix"]             = { strRecipient = "Leader",    strComment = "Crafting" },
							["Tarnished Eldan Gift"]       = { strRecipient = "Randomize", strComment = "" },
							["Encrypted Datashard"]        = { strRecipient = "Leader",    strComment = "Crafting" },
						}
					},
					["Default - Raid GA iLvl 96"] = {
						bIsDefault = true,
						bIsLoadable = false,
						nCreatedAt = 0,
						strDisplayName = "N/A",
						tLootList = {
							["Inoculation Resonators"]         = { strRecipient = "Randomize",  strComment = "Support Resonators" },
							["Hyperproliferative Denims"]      = { strRecipient = "Randomize",  strComment = "Heavy Assault Chest" },
							["Rib-Boned Breastplate"]          = { strRecipient = "Randomize",  strComment = "Heavy Assault Chest" },
							["Poly-Tendered Waders"]           = { strRecipient = "Randomize",  strComment = "Heavy Assault Legs" },
							["Insertional Stamina Gene"]       = { strRecipient = "Randomize",  strComment = "Assault Weapon Attachment" },
							["Calcified Longbone"]             = { strRecipient = "Randomize",  strComment = "Assault Sword" },
							["Scattershot Dispersion Blaster"] = { strRecipient = "Randomize",  strComment = "Support Heavy Gun" },
							["Overspecialized Regenerators"]   = { strRecipient = "Randomize",  strComment = "Medium Assault Hands" },
							["Targeted Viral Coat"]            = { strRecipient = "Randomize",  strComment = "Light Assault Chest" },
							["Dormant Phase Duster"]           = { strRecipient = "Randomize",  strComment = "Light Support Chest" },
							["Meningeal Vest"]                 = { strRecipient = "Randomize",  strComment = "Light Assault Chest" },
							["Biomimetic Slashers"]            = { strRecipient = "Randomize",  strComment = "Support Claws" },
							["Curved Strain Phalanges"]        = { strRecipient = "Randomize",  strComment = "Support Support System" },
							["Genetic Redundancy Revolvers"]   = { strRecipient = "Randomize",  strComment = "Support Pistols" },
							["Hallucinatory Secretions"]       = { strRecipient = "Randomize",  strComment = "Assault Implant" },
						}
					},
					["Default - Raid DS iLvl 116"] = {
						bIsDefault = true,
						bIsLoadable = false,
						nCreatedAt = 0,
						strDisplayName = "N/A",
						tLootList = {
							["Crystalline Multiphasic Slicers"]    = { strRecipient = "Randomize",  strComment = "Assault Claws" },
							["Intrusive Espernetic Technopants"]   = { strRecipient = "Randomize",  strComment = "Light Assault Legs" },
							["Sympathetic Espernetic Neuroblade"]  = { strRecipient = "Randomize",  strComment = "Support Psyblade" },
							["Stellar Binarymesh Shoulder Boards"] = { strRecipient = "Randomize",  strComment = "Heavy Support Shoulders" },
							["Electrolined Handscapes"]            = { strRecipient = "Randomize",  strComment = "Heavy Assault Hands" },
							["Dynamic Onslaught Terminal"]         = { strRecipient = "Randomize",
								strComment = "Support Weapon Attachment" },
							["Stellar Binarymesh Gravblade"]       = { strRecipient = "Randomize",  strComment = "Support Sword" },
							["Phage-Clogged Howitzer"]             = { strRecipient = "Randomize",  strComment = "Assault Heavy Gun" },
							["Biomimetic Receptive Cowl"]          = { strRecipient = "Randomize",  strComment = "Medium Support Helm" },
							["Invasive Biomimetic Zapinators"]     = { strRecipient = "Randomize",  strComment = "Assault Resonators" },
							["Biomimetic Receptive Zapinators"]    = { strRecipient = "Randomize",  strComment = "Support Resonators" },
							["Crystalline Multiphasic Flexisuit"]  = { strRecipient = "Randomize",  strComment = "Medium Assault Chest" },
							["Defiant Multiphasic Visor"]          = { strRecipient = "Randomize",  strComment = "Medium Support Helm" },
							["Heart of the Claw"]                  = { strRecipient = "Randomize",  strComment = "Light Assault Chest" },
							["Gloomsheath"]                        = { strRecipient = "Randomize",  strComment = "Light Support Chest" },
              ["Precision Heavy-Bore Painmakers"]    = { strRecipient = "Randomize",  strComment = "Assault Pistols" },
							["Intrusive Augmentor"]                = { strRecipient = "Randomize",  strComment = "Assault Support System" },
							["Protective Multiphasic Deflector"]   = { strRecipient = "Randomize",  strComment = "Tank Shield" },
							["Defiant Multiphasic Deflector"]      = { strRecipient = "Randomize",  strComment = "Tank Shield" },
						}
					},
					["Default"] = {
						bIsDefault = true,
						bIsLoadable = true,
						strDisplayName = "Default",
						nCreatedAt = 0,
						tLootList = {}
					}
				},
				tAutoloot = {
					tProfileRef = nil,
					strProfileName = "Default",
				},
				tLootDetails = {},
			},
		},
		tSettings = {},
		tNewSettings = nil,
		tCallbacks = {},
		tLinker = G.Linker:New()
	}
	o.tDefaults.tData.tAutoloot.tProfileRef = o.tDefaults.tData.tAutolootProfiles["Default"]

	setmetatable(o, self)
	self.__index = self
	return o
end

function Settings:Init(tTarget)
	self.tLinker:Link(NLLC, self)
end

function Settings:Destroy()
	self.tLinker:Destroy()
	self.tLinker = nil
	self.tCallbacks = nil
end

----------------------------------------------------------------------------------
-- Functions
----------------------------------------------------------------------------------
function Settings:Save(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
    self:PurgeSettings()
	return self.tSettings
end

function Settings:PurgeSettings()
	if self.tSettings.tData == nil then
		return
	end

	-- No need to save this data again.
	if self.tSettings.tData.tAutolootProfiles ~= nil then
		for strKey, tProfile in pairs(self.tSettings.tData.tAutolootProfiles) do
			if not tProfile.bIsLoadable then
				self.tSettings.tData.tAutolootProfiles[strKey] = nil
			end
		end
	end

	-- This is used internally and should not be presisted.
	if self.tSettings.tData.tAutoloot ~= nil then
		self.tSettings.tData.tAutoloot.tProfileRef = nil
	end

	if not G.Utils:IsInRaid() or G.Debug.bIsTest then
		self.tSettings.tData.tLootDetails = nil
	end
end

function Settings:Load(tData)

	-- Start with default settings.
	self.tSettings = G.Utils:Deepcopy(self.tDefaults)

	if tData ~= nil then
		-- Make a copy of saved settings only if they exists.
		self.tNewSettings = G.Utils:Deepcopy(tData)
		self.bIsNewVersion = G.CURRENT_VERSION ~= self.tNewSettings["strVersion"]

		-- We call RestoreSettings to make sure we only restore currently known
		-- settings. That is we should only care about settings currently in
		-- the defaults objects and discared others.
		self:RestoreSettings(self.tSettings, self.tNewSettings)

		-- Restore data settings
		self:RestoreDataSettings(self.tSettings.tData, self.tDefaults.tData)
	end
end

function Settings:Reset()
	self.tSettings = G.Utils:Deepcopy(self.tDefaults)
	G.Debug:OnRestore()
end

function Settings:ReadSetting(strKeySequence, default)
	local strKeys = G.Utils:Explode(".", strKeySequence)
	local nNumKeys = #strKeys
	local tSettings = self.tSettings

	for nIndex, strKey in ipairs(strKeys) do
		if nIndex == nNumKeys then
			local setting = tSettings[strKey]
			if setting == nil then
				return default
			else
				return setting
			end
		else
			tSettings = tSettings[strKey]
			if tSettings == nil then
				return default
			end
		end
	end

	return default
end

----------------------------------------------------------------------------------
-- Getters & Helpers
----------------------------------------------------------------------------------
function Settings:GetDataBlock() self:ReadSetting("tData") end
function Settings:ReadBlock(strKeySequence, default) return self:ReadSetting("tData."..strKeySequence, default) end
function Settings:Read(strKeySequence, default) return self:ReadSetting(strKeySequence, default) end
function Settings:GetTable() return self.tSettings end
function Settings:GetDefaultTable() return self.tDefaults end
function Settings:GetNewTable() return self.tNewSettings end

----------------------------------------------------------------------------------
-- Private functions
----------------------------------------------------------------------------------
function Settings:RestoreSettings(tCurrentSettings, tNewSettings, bIsDataStorage)
	for strKey, tCurrentValue in pairs(tCurrentSettings) do
		local tNewValue = tNewSettings[strKey]

		if tNewValue ~= nil then
			if not bIsDataStorage and type(tNewValue) == 'table' then
				self:RestoreSettings(tCurrentValue, tNewValue, strKey == "tData")
			else
				if bIsDataStorage then
					tCurrentSettings[strKey] = tNewValue -- Copy the table ref
				else
					if tCurrentSettings[strKey] ~= nil and strKey ~= "strVersion" then
						tCurrentSettings[strKey] = tNewValue
					end
				end
			end
		end
	end
end

function Settings:RestoreDataSettings(tData, tDefaultData)
	-- Default autoprofiles must be copied because they are purged before saving.
	-- Note: We should exlude the "Default" profile as that is the starting non-removable profile
	--       which the user can make changes to and thus is save.
	for strKey, tProfile in pairs(tDefaultData.tAutolootProfiles) do
		if not tProfile.bIsLoadable or tData.tAutolootProfiles[strKey] == nil then
			tData.tAutolootProfiles[strKey] = G.Utils:Deepcopy(tProfile)
		end
	end
end
