local Apollo = require "Apollo"
local ChatSystemLib = require "ChatSystemLib"

local NLLC = Apollo.GetAddon("NextLabLootClient")

local RollWatcher = Apollo.GetPackage("Gemini:Hook-1.0").tPackage:Embed({})
NLLC.RollWatcher = RollWatcher
Apollo.LinkAddon(NLLC, RollWatcher)

local Ranker = NLLC.Ranker

NLLC:OnceDocLoaded(function()
	local Jita = Apollo.GetAddon("Jita")
	local ChatLog = Apollo.GetAddon("ChatLog")
	if Jita then
		local Client = Jita.Client
		RollWatcher:RawHook(Client, "OnChatMessage", "OnMessage")
		function RollWatcher.hookedFunc(...)
			return RollWatcher.hooks[Client]["OnChatMessage"](Client, ...)
		end
	elseif ChatLog then
		RollWatcher:RawHook(ChatLog, "OnChatMessage", "OnMessage")
		function RollWatcher.hookedFunc(...)
			return RollWatcher.hooks[ChatLog]["OnChatMessage"](ChatLog, ...)
		end
	end
end)

function RollWatcher:DeepCopy(arg)
	if type(arg) == "table" then
		local tbl = {}
		for i,v in pairs(arg) do
			tbl[self:DeepCopy(i)] = self:DeepCopy(v)
		end
		return tbl
	else
		return arg
	end
end

function RollWatcher:OnMessage(this, channel, message, ...)
	if channel:GetType() == ChatSystemLib.ChatChannel_System then
		local strMessage = ""
		for _, tSegment in ipairs(message.arMessageSegments) do
			strMessage = strMessage..tSegment.strText
		end

		local name, roll, rollMax = strMessage:match("^([^%s]+%s[^%s]+)%s.*%s(%d+)%s%(1%-(%d+)%)%s$")
		roll, rollMax = tonumber(roll), tonumber(rollMax)
		if name and roll and rollMax and (rollMax == 100 or rollMax == 50) then
			local nRank = Ranker:GetLootRank(name, rollMax == 100)
			if nRank ~= nil then --nRank can be false, for 'does not have offspec'
				message = self:DeepCopy(message) --this is to prevent other addons from seeing this event with our changed msg
				table.insert(message.arMessageSegments, 1, {
					bAlien = false,
					bProfanity = false,
					bRolePlay = false,
					strText = "<"..(nRank or "-").."> ",
				})
			end
		end
	end
	return self.hookedFunc(channel, message, ...)
end
