require "Window"
local Apollo = require "Apollo"
local XmlDoc = require "XmlDoc"
local GameLib = require "GameLib"
local ApolloTimer = require "ApolloTimer"

-----------------------------------------------------------------------------------------------
-- NextLabLootClient Module Definition
-----------------------------------------------------------------------------------------------
local NLLC = {}
local AddonName = "NextLabLootClient"

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function NLLC:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    -- initialize variables here

    return o
end

function NLLC:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = AddonName
	local tDependencies = {}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function NLLC:LoadForm(name, parent, handler)
	return Apollo.LoadForm(self.xmlDoc, name, parent, handler)
end

local errorCache = {}
local function reportError(strError)
  local strTraceback = debug.traceback(strError, 2)
  table.insert(errorCache, strTraceback)
end

function NLLC:OnDelayedDocLoadTimer() --triggered in :OnDocLoaded - 2s Timer
  self:OnDelayedRestore()

  for i, strTraceback in ipairs(errorCache) do
    Apollo.AddAddonErrorText(AddonName, strTraceback)
    Event_FireGenericEvent("LuaError", Apollo.GetAddonInfo(AddonName), strTraceback, true)
  end
  errorCache = nil

  reportError = function(strError)
    local strTraceback = debug.traceback(strError, 2)
    Apollo.AddAddonErrorText(AddonName, strTraceback)
    Event_FireGenericEvent("LuaError", Apollo.GetAddonInfo(AddonName), strTraceback, true)
  end
end

-----------------------------------------------------------------------------------------------
-- NextLabLootClient OnLoad
-----------------------------------------------------------------------------------------------
do
	local load = {}
	local loaded = false

	local function exec(f, t)
    xpcall(function()
      if type(f) == "function" then
        f()
      elseif type(t[f]) == "function" then
        t[f](t)
      end
    end, reportError)
	end

	function NLLC:OnLoad()
		-- load our form file
		NLLC = self
		self.xmlDoc = XmlDoc.CreateFromFile("Forms.xml")
		self.xmlDoc:RegisterCallback("OnDocLoaded", self)

		--excecute all those who asked for it.
		for t,f in pairs(load) do
			exec(f,t)
		end
	end

	function NLLC:OnceLoaded(func, tbl)
		if not func then return end

		if not loaded then
			if type(tbl) == "table" then
				load[tbl] = func --if tbl is actually a table, we assume func is a correct key for it.
			elseif type(func) == "function" then
				load[#load+1] = func
			end --all other situations are useless.
		else
			if type(tbl) == "table" then
				if type(tbl[func]) == "function" then
					tbl[func](tbl)
				end
			elseif type(func) == "function" then
				func()
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- NextLabLootClient OnDocLoaded
-----------------------------------------------------------------------------------------------
do
	local load = {}
	local loaded = false

	local function exec(f, t)
    xpcall(function()
      if type(f) == "function" then
        f()
      elseif type(t[f]) == "function" then
        t[f](t)
      end
    end, reportError)
	end

	function NLLC:OnDocLoaded()
		loaded = true

		for t,f in pairs(load) do
			exec(f,t)
		end

    self.delayedDocLoadTimer = ApolloTimer.Create(2, false, "OnDelayedDocLoadTimer", self)
	end

	function NLLC:OnceDocLoaded(func, tbl)
		if not func then return end

		if not loaded then
			if type(tbl) == "table" then
				load[tbl] = func --if tbl is actually a table, we assume func is a correct key for it.
			elseif type(func) == "function" then
				load[#load+1] = func
			end --all other situations are useless.
		else
			if type(tbl) == "table" then
				if type(tbl[func]) == "function" then
					tbl[func](tbl)
				end
			elseif type(func) == "function" then
				func()
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- NextLabLootClient OnRestore
-----------------------------------------------------------------------------------------------
do
	local restore = {}
	local restored = {}
  local maxRestore = 0

	local function exec(f, t, eType, tSavedData)
    xpcall(function()
      if type(f) == "function" then
        f(eType, tSavedData)
      elseif type(t[f]) == "function" then
        t[f](t, eType, tSavedData)
      end
    end, reportError)
	end

	function NLLC:OnRestore(eType, tSavedData)
		restored[eType] = tSavedData

    for i = 1, maxRestore, 1 do
      local tbl = restore[i]
      if tbl then
        local f, t = unpack(tbl)
        exec(f,t,eType,tSavedData)
      end
		end
	end

	function NLLC:OnceRestored(func, tbl)
		if not func then return end
    local index;

    if type(tbl) == "table" or type(func) == "function" then
      local i = #restore+1
      if i>maxRestore then maxRestore = i end
      restore[i] = {func, tbl}
      index = i
    else
      local a,b = type(func), type(tbl)
      reportError(("Wrong arguements for 'OnceRestored': #1 function (is: %s) or #2 table (is: %s)"):format(a,b))
      return nil
    end

    for eType, tSavedData in pairs(restored) do
      if type(tbl) == "table" then
        exec(func, tbl, eType, tSavedData)
			elseif type(func) == "function" then
				exec(func, tbl, eType, tSavedData)
			end
    end
    return index
	end

  function NLLC:RemoveOnceRestored(key)
    if key and restore[key] then
      restore[key] = nil
    end
  end

  --now: a little bit of hacking to make OnceRestored always trigger for every eType, even if not supplied by the game.
  function NLLC:OnDelayedRestore()
    for _, eType in pairs(GameLib.CodeEnumAddonSaveLevel) do
      if not restored[eType] then
        self:OnRestore(eType, false)
      end
    end
  end
end

-----------------------------------------------------------------------------------------------
-- NextLabLootClient OnSave
-----------------------------------------------------------------------------------------------
do
	local save = {}
  local maxSave = 0

	local function exec(f, t, eType, data)
    xpcall(function()
      if type(f) == "function" then
        f(eType, data)
      elseif type(t[f]) == "function" then
        t[f](t, eType, data)
      end
    end, reportError)
	end

	function NLLC:OnSave(eType)
    local data = {}

    for i = 1, maxSave, 1 do
      local tbl = save[i]
      if tbl then
        local f,t = unpack(tbl)
        exec(f,t,eType,data)
      end
		end

    return data
	end

	function NLLC:OnceSaving(func, tbl)
		if not func then return end

    if type(tbl) == "table" or type(func) == "function" then
      local i = #save+1
      if i>maxSave then maxSave = i end
      save[i] = {func, tbl}
      return i
    else
      local a,b = type(func), type(tbl)
      reportError(("Wrong arguements for 'OnceSaving': #1 function (is: %s) or #2 table (is: %s)"):format(a,b))
    end
	end

--this only works if you know the key (return value of :OnceSaving) Do not do this more than once!
  function NLLC:RemoveOnceSaving(key)
    if key and save[key] then
      save[key] = nil
    end
  end
end

-----------------------------------------------------------------------------------------------
-- NextLabLootClient Instance
-----------------------------------------------------------------------------------------------
local NLLCInst = NLLC:new()
NLLCInst:Init()
