local Apollo = require "Apollo"
local ApolloColor = require "ApolloColor"

local NLLC = Apollo.GetAddon("NextLabLootClient")
local UIComponents = {}
NLLC.UIComponents = UIComponents
Apollo.LinkAddon(NLLC, UIComponents)

local Options = NLLC.Options

local FORM_DROPDOWN_TEMPLATE = "Dropdown"
local FORM_DROPDOWN_ITEM = "DropdownItem"
local FORM_SLIDER_TEMPLATE = "Slider"
local FORM_CHECKBOX_TEMPLATE = "Checkbox"
local FORM_TEXTBOX_TEMPLATE = "TextBox"
local FORM_COLORPICKER_TEMPLATE = "ColorPicker"
local FORM_COLORBUTTON_TEMPLATE = "ColorButton"
local FORM_POSITIONBOARD_TEMPLATE = "PositionBoard"
local FORM_FONTBOX_TEMPLATE = "FontBox"
local FORM_SPRITEBOX_TEMPLATE = "SpriteSelector"
local FORM_SPRITELIST_TEMPLATE = "SpriteList"
local FORM_SPRITELINE_TEMPLATE = "SpriteLine"

do--[[#####  PositionBoard  ######]]
	local PositionBoard = {}
	PositionBoard.__index = PositionBoard

	function UIComponents:applyPositionBoard(parent, parentOpt, xIdx, yIdx, zIdx)
		local o = setmetatable({}, PositionBoard)
		o.board = NLLC:LoadForm(FORM_POSITIONBOARD_TEMPLATE, parent, o)
		o.grid = o.board:FindChild("Grid")
		o.thumb = o.board:FindChild("Grid:Thumb")
		o.slider = o.board:FindChild("SliderBG:SliderBar")
		o.xTxt = o.board:FindChild("Text_xPos")
		o.yTxt = o.board:FindChild("Text_yPos")
		o.zTxt = o.board:FindChild("Text_zPos")
		o.xPosOpt = Options:GetOption(parentOpt, xIdx)
		o.yPosOpt = Options:GetOption(parentOpt, yIdx)
		o.zPosOpt = Options:GetOption(parentOpt, zIdx)
		o.xPosOpt:OnUpdate(o, "UpdateXPos")
		o.yPosOpt:OnUpdate(o, "UpdateYPos")
		o.zPosOpt:OnUpdate(o, "UpdateZPos")

		o.xmax = math.floor(o.grid:GetWidth()/2)-15
		o.ymax = math.floor(o.grid:GetHeight()/2)-15
		o.xmin = -o.xmax-30
		o.ymin = -o.ymax-30

		o.blockGrid = false
		o.blockSlid = false
		o.blockTxtX = false
		o.blockTxtY = false
		o.blockTxtZ = false

		o.xCord = 0
		o.yCord = 0
		o.zCord = 0
	end

	function PositionBoard:UpdateXPos(val)
		if not self.blockGrid then
			self.xCord = val
		end
		if not self.blockTxtX then
			self.xTxt:SetText(tostring(val))
		end
	end

	function PositionBoard:UpdateYPos(val)
		if not self.blockSlid then
			self.yCord = val
		end
		if not self.blockTxtY then
			self.yTxt:SetText(tostring(val))
		end
	end

	function PositionBoard:UpdateZPos(val)
		if not self.blockGrid then
			self.zCord = val
		end
		if not self.blockTxtZ then
			self.zTxt:SetText(tostring(val))
		end
	end

	function PositionBoard:ThumbButtonDown(wndControl, wndHandler, arg1, arg2)
		self.mouseRef = Apollo.GetMouse()
		self.thumbRefX, self.thumbRefY = self.thumb:GetAnchorOffsets()
		self.blockGrid = true
		Apollo.RegisterEventHandler("NextFrame", "CheckMouse", self)
	end

	function PositionBoard:ThumbButtonUp(wndControl, wndHandler, arg1, arg2)
		if not self.blockGrid then return end
		Apollo.RemoveEventHandler("NextFrame", self)
		self.thumb:SetAnchorOffsets(-15, -15, 15, 15)
		self.blockGrid = false
		self.xPosOpt:ForceUpdate()
		self.zPosOpt:ForceUpdate()
	end

	function PositionBoard:GridButtonDown(wndControl, wndHandler, arg1, xPos, yPos)
		self.mouseRef = Apollo.GetMouse()
		self.thumbRefX, self.thumbRefY = xPos-self.xmax-30, yPos-self.ymax-30
		self.blockGrid = true
		Apollo.RegisterEventHandler("NextFrame", "CheckMouse", self)
	end

	function PositionBoard:GridButtonUp(wndControl, wndHandler, ...)
		if not self.blockGrid then return end
		Apollo.RemoveEventHandler("NextFrame", self)
		self.thumb:SetAnchorOffsets(-15, -15, 15, 15)
		self.blockGrid = false
		self.xPosOpt:ForceUpdate()
		self.zPosOpt:ForceUpdate()
	end

	function PositionBoard:CheckMouse()
		local m = Apollo.GetMouse()
		local x, y = m.x-self.mouseRef.x+self.thumbRefX, m.y-self.mouseRef.y+self.thumbRefY
		x = math.max(self.xmin, math.min(x, self.xmax))
		y = math.max(self.ymin, math.min(y, self.ymax))

		self.thumb:SetAnchorOffsets(x, y, x+30, y+30)

		x = math.floor(((x+15)/16)+0.5)
		y = math.floor(((y+15)/16)+0.5)
		local a, b = self.xCord+x, self.zCord+y
		if a ~= self.xPosOpt:Get() then
			self.xPosOpt:Set(a)
		end
		if b ~= self.zPosOpt:Get() then
			self.zPosOpt:Set(b)
		end
	end

	function PositionBoard:CheckSlider()
		if not self.slider:IsThumbDragging() then
			Apollo.RemoveEventHandler("NextFrame", self)
			self.slider:SetValue(0)
			self.blockSlid = false
			self.yPosOpt:ForceUpdate()
		end
	end

	function PositionBoard:YSliderChanged(wndControl, wndHandler, val)
		self.blockSlid = true
		Apollo.RegisterEventHandler("NextFrame", "CheckSlider", self)
		val = math.floor((self.ymax*val/15)+0.5)
		self.yPosOpt:Set(self.yCord+val)
		--self.blockSlid = false --from inside checkSlider
	end

	function PositionBoard:OnTextXPos(wndControl, wndHandler, txt)
		local num = tonumber(txt)
		if num then
			self.blockTxtX = true
			self.xPosOpt:Set(math.floor(num))
			self.blockTxtX = false
		end
	end

	function PositionBoard:OnTextYPos(wndControl, wndHandler, txt)
		local num = tonumber(txt)
		if num then
			self.blockTxtY = true
			self.yPosOpt:Set(math.floor(num))
			self.blockTxtY = false
		end
	end

	function PositionBoard:OnTextZPos(wndControl, wndHandler, txt)
		local num = tonumber(txt)
		if num then
			self.blockTxtZ = true
			self.zPosOpt:Set(math.floor(num))
			self.blockTxtZ = false
		end
	end
end

do--[[#####  Dropdown  ######]] --:applyDropdown(parent, choices, selector, translator, ...)
	local function toggleDropdown(self, wndHandler, wndControl)
		if wndHandler ~= wndControl then
			return
		end

		local container = wndControl:FindChild("ChoiceContainer")
		local choices = container:FindChild("Choices")
		if wndControl:IsChecked() then
			local frames = choices:GetChildren()

			local pre = #frames --number of previously created Frames
			local num = 0 --number of items to be displayed

			--Fill Space:
			for i, obj in self.ipairs(self.data) do
				local text = self.trans(obj)
				if frames[i] then
					frames[i]:SetText(text)
					frames[i]:SetData({i, obj, self})
				else
					frames[i] = NLLC:LoadForm(FORM_DROPDOWN_ITEM, choices, self)
					frames[i]:SetText(text)
					frames[i]:SetData({i, obj, self})
				end
				num = i
			end
			for i = num+1, #frames, 1 do
				frames[i]:Destroy()
			end

			choices:ArrangeChildrenVert()


			--Make Enought Space for the created Items:
			local l,t,r,b = container:GetAnchorOffsets()
			--32 = Size of a single Item
			pre = pre < 10 and pre or 10 --we didnt display more than 10 items
			num = num < 10 and num or 10 --we do not want to display more than 10 items
			local diff = (pre - num)*25
			b = b - diff --growing = going lower
			container:SetAnchorOffsets(l,t,r,b)
		end
		container:Show(wndControl:IsChecked())
	end

	local function containerClosed(self, container)
		local btn = container:GetParent()
		btn:SetCheck(false)
	end

	local function setSelected(self, selected)
		self.sel:Set(selected)
	end

	local function clickChoice(self, btn)
		local index, data = unpack(btn:GetData())

		local container = btn:GetParent():GetParent()

		container:Close()

		setSelected(self, data, index)
	end

	local function setIndex(self, index)
		for i,v in self.ipairs(self.data) do
			if i == index then
				return setSelected(self, v)
			end
		end
	end

	local function registerForDistantUpdates(handler, x,...)
		if not x then return end
		x:OnUpdate(handler, "OnDistantUpdate")
		registerForDistantUpdates(handler, ...)
	end

	local function setText(self, str)
		self.drop:SetText(str)
	end

	local function selectorUpdate(self, value)
		self:SetText(self.trans(value))
	end

	local function distantUpdate(handler)
		handler:SetText(handler.trans(handler.sel:Get()))
	end

	local function transToFunction(input)
		local t = type(input)
		if t=="function" then
			return input
		elseif t=="table" then
			return function(val)
					return input[val or false] or input[false] or " - "
				end
		else
			return tostring
		end
	end

	--parent will be completely filled with the dropdown.
	--choices is either a array, or a object with a :ipairs() method. (only ipairs(c)/c:ipairs() will be used to gain
	--		access to elements in the choices)
	--selector (of type Option) will be :Set() with a value from choices, when the user selects something in the
	--		Dropdown - it also supplies the Dropdown with the Object to display as text
	--translator is a function which accepts the value from the selector and returns a String. Default: tostring()
	--... are addtional Options, to provide more Updates. any recieved :OnUpdate() results in retexting the
	--		dropdown - nothing else.
	--The Text on the Dropdown will only change whenever the selector or one of the other options push an Update
	--if you do not want to display Text - just rewrite the :SetText(str) function.
	function UIComponents:applyDropdown(parent, choices, selector, translator, ...)
		local handler = {
			data = choices,
			trans = transToFunction(translator),
			sel = selector,
			ipairs = choices.ipairs or ipairs,
		}

		handler.drop = NLLC:LoadForm(FORM_DROPDOWN_TEMPLATE, parent, handler):FindChild("DropdownButton")

		handler.OnDropdownToggle = toggleDropdown
		handler.OnContainerClosed = containerClosed
		handler.OnItemSelected = clickChoice

		handler.OnSelectorUpdate = selectorUpdate
		handler.OnDistantUpdate = distantUpdate
		handler.SetSelectedIndex = setIndex
		handler.SetSelectedValue = setSelected
		handler.SetText = setText

		selector:OnUpdate(handler, "OnSelectorUpdate")

		registerForDistantUpdates(handler, ...)

		return handler
	end
end

do--[[#####  Slider  #####]] --:applySlider(parent, selector, min, max, steps, tbNoSteps, tbNoLimit, tbNoLimitPos)
	local function sliderChanged(self, wndHandler, wndControl, value, old)
		--somehow the value is not exactly on point.. RECALCUALTE!
		local ticks = math.floor(((value-self.minimum)/self.ticklen)+0.5)
		value = self.minimum + self.ticklen*ticks

		self.block = true --uncool to have the bar be set all the time when dragging it.
		self.opt:Set(value)
		self.block = false
	end

	local function onUpdate(handler, val)
		if not handler.block then
			handler.slider:SetValue(val or handler.minimum)
		end
		if not handler.blockTxt then
			handler.text:SetText(tostring(val))
		end
	end

	local function setMinMax(handler, min, max, step)
		handler.minimum = min or handler.minimum
		handler.maximum = max or handler.maximum
		handler.ticklen = step or handler.ticklen

		if handler.minimum > handler.maximum then --keep at least one value possible for the slider.
			if min and not max then
				handler.maximum = min
			elseif max and not min then
				handler.minimum = max
			end
		end

		handler.slider:SetMinMax(handler.minimum, handler.maximum, handler.ticklen)
	end

	local function onText(self, wndHandler, wndControl, txt)
		local value = tonumber(txt)
		if not value then return end --non numeric values are not interesting.

		if self.tbLimited then
			if value < self.minimum then
				value = self.minimum
			elseif value > self.maximum and self.tbPosLimit then
				value = self.maximum
			end
		end

		if self.tbSteps then
			local ticks = math.floor(((value-self.minimum)/self.ticklen)+0.5)
			value = self.minimum + self.ticklen*ticks
		end

		self.blockTxt = true
		self.opt:Set(value)
		self.blockTxt = false
	end

	local function lostFocus(self)
		self.opt:ForceUpdate() --put value onto Slider & Textbox
	end

	--tbNoSteps, tbNoLimit -> the textbox ignores Steps, Min&Max
	function UIComponents:applySlider(parent, selector, min, max, steps, tbNoSteps, tbNoLimit, tbNoLimitPos)
		min = min or 0
		max = max or min+1
		steps = steps or 0.1

		local handler = {opt = selector, block = false, minimum = min, maximum = max, ticklen = steps,
						tbSteps = not tbNoSteps, tbLimited = not tbNoLimit, tbPosLimit = not tbNoLimitPos}
		handler.SliderChanged = sliderChanged
		handler.OnUpdate = onUpdate
		handler.SetMinMax = setMinMax
		handler.OnText = onText
		handler.LostFocus = lostFocus
		handler.form = NLLC:LoadForm(FORM_SLIDER_TEMPLATE, parent, handler)
		handler.text = handler.form:FindChild("Label")
		handler.slider = handler.form:FindChild("SliderBar")

		handler.slider:SetMinMax(min, max, steps)

		selector:OnUpdate(handler, "OnUpdate")
		return handler
	end
end

do--[[#####  Checkbox #####]] --:applyCheckbox(parent, selector, text)
	local function onCheck(handler)
		if handler.block then return end
		handler.block = true
		handler.sel:Set(true)
		handler.block = false
	end

	local function onUncheck(handler)
		if handler.block then return end
		handler.block = true
		handler.sel:Set(false)
		handler.block = false
	end

	local function updateCheck(handler, newVal)
		if handler.block then return end
		handler.block = true
		handler.form:SetCheck(newVal and true or false)
		handler.block = false
	end

	function UIComponents:applyCheckbox(parent, selector, text)
		local handler = {
			Check = onCheck,
			Uncheck = onUncheck,
			OnUpdate = updateCheck,
			block = false,
			sel = selector,
		}
		handler.form = NLLC:LoadForm(FORM_CHECKBOX_TEMPLATE, parent, handler)
		handler.form:SetText(text)

		selector:OnUpdate(handler, "OnUpdate")
		return handler
	end
end

do--[[#####  Textbox #####]] --:applyTextbox(parent, selector)
	local function textChanged(handler, _, _, str)
		if handler.block then return end
		handler.block = true
		handler.sel:Set(str)
		handler.block = false
	end

	local function updateText(handler, newVal)
		if handler.block then return end
		handler.block = true
		handler.text:SetText(newVal or "")
		handler.block = false
	end

	local function lostFocus(handler)
		handler.text:SetText(tostring(handler.sel:Get() or ""))
	end

	function UIComponents:applyTextbox(parent, selector, textcolor)
		local handler = {
			OnTextChange = textChanged,
			OnLostFocus = lostFocus,
			OnUpdate = updateText,
			block = false,
			sel = selector,
		}
		handler.form = NLLC:LoadForm(FORM_TEXTBOX_TEMPLATE, parent, handler)
		handler.text = handler.form:FindChild("Text")

		if textcolor then
			handler.text:SetTextColor(textcolor)
		end

		selector:OnUpdate(handler, "OnUpdate")
		return handler
	end
end

do--[[#####  Colorbutton #####]] --:applyColorbutton(parent, selector, asTbl)
	local white = ApolloColor.new(1,1,1,1)
	local picker = nil

	local function chooseNew(self, wndHandler, wndControl)
		--if not wndControl:IsMouseTarget() then
		--	local target = Apollo.GetMouseTargetWindow()
		--	if target:GetName() == FORM_DROPDOWN_ITEM then
		--		local handler = select(3, unpack(target:GetData()))
		--		handler:OnItemSelected(target)
		--	end
		--	return
		--end
		local init = self.sel:Get() or white

		picker:Pick(self, init)
	end

	local function tblCallback(handler, t, str)
		handler.sel:Set(t)
	end

	local function strCallback(handler, t, str)
		handler.sel:Set(str)
	end

	local function updateOpt(handler, newVal)
		handler.colorBG:SetBGColor(ApolloColor.new(newVal or white))
	end

	function UIComponents:applyColorbutton(parent, selector, asTbl)
		if not picker then picker = UIComponents:InitColorPicker() end --dont check if already, its built that way.

		local handler = {sel = selector, ChooseColor = chooseNew, OnUpdate = updateOpt}
		handler.Set = asTbl and tblCallback or strCallback

		local form = NLLC:LoadForm(FORM_COLORBUTTON_TEMPLATE, parent, handler)
		handler.colorBG = form:FindChild("Button:Color")

		handler.Set = asTbl and tblCallback or strCallback

		selector:OnUpdate(handler, "OnUpdate")

		--selector:ForceUpdate()
	end

	function UIComponents:InitColorPicker()
		local L = NLLC:Localize({--[[English]]
			["tt2x"] = [[WildStar supports colors with twice as high color-values.
				These Colors are marked as '2x'. With this you can (for example) apply bright colors to gray textures.]],
		}, {--[[German]]
			["Color Picker"] = "Farbauswahl",
			["New Color:"] = "Neue Farbe:",
			["Old Color:"] = "Alte Farbe:",
			["Copy"] = "Kopieren",
			["Paste"] = "Einf�gen",
			["tt2x"] = [[WildStar unterstützt Farben mit doppelten Farbwerten. Diese Farben werden mit '2x' markiert.
				Auf diese Weise kann zum Beispiel eine helle Farbe auf eine graue Textur gelegt werden.]],
		}, {--[[French]]})

		local colorHandler = {color = ApolloColor.new(1,1,1,1), alpha = 1, copied = ApolloColor.new(1,1,1,1)}
		local colorPicker = nil
		function colorHandler:OnColorChanged( wndHandler, wndControl, color )
			local r = ("%x"):format(color.r*255):gsub("^(%x)$", "0%1")
			local g = ("%x"):format(color.g*255):gsub("^(%x)$", "0%1")
			local b = ("%x"):format(color.b*255):gsub("^(%x)$", "0%1")
			local a = ("%x"):format(self.alpha*255):gsub("^(%x)$", "0%1")
			local pre = self.dbl and "2x:" or ""

			local full = pre..a..r..g..b

			color = ApolloColor.new(a..r..g..b)

			self.color = color

			self:SetRText(r)
			self:SetGText(g)
			self:SetBText(b)
			self:SetAText(a)
			self:SetAlphaSlider(color.a)
			self:Set2x(self.dbl)

			self.newColor:SetBGColor(ApolloColor.new(full))
			if self.callback then
				self.callback:Set({a=color.a, r=color.r, g=color.g, b=color.b}, full)
			end
		end

		function colorHandler:AlphaSliderChanged(_,_, val)
			self.blockAlpha = true
			self.alpha = val
			self:OnColorChanged(self.picker, self.picker, self.color)
			self.blockAlpha = false
		end

		function colorHandler:SetAlphaSlider(a)
			if self.blockAlpha then return end
			self.slider:SetValue(a)
		end

		function colorHandler:Check2x()
			self.dbl = true
			self:OnColorChanged(self.picker, self.picker, self.color)
		end

		function colorHandler:Uncheck2x()
			self.dbl = false
			self:OnColorChanged(self.picker, self.picker, self.color)
		end

		function colorHandler:Set2x(val)
			self.btn2x:SetCheck(val)
		end


		local blockA = false
		function colorHandler:ColorChangedA( wndHandler, wndControl, strText )
			local str = strText:sub(1,2) --ignore all other
			local num = tonumber(str, 16) --interpret as number in hex.
			if not num then return end --if the Text does not represent a number - cancel overwriting

			blockA = true
			self.alpha = num/255
			self:OnColorChanged(self.picker, self.picker, self.color)
			blockA = false
		end

		function colorHandler:SetAText(str)
			if blockA then return end
			self.boxA:SetText(str)
		end

		local blockR = false
		function colorHandler:ColorChangedR( wndHandler, wndControl, strText )
			local str = strText:sub(1,2) --ignore all other
			local num = tonumber(str, 16) --interpret as number in hex.
			if not num then return end --if the Text does not represent a number - cancel overwriting

			blockR = true
			local newColor = ApolloColor.new(num/255, self.color.g, self.color.b, self.color.a)
			self.picker:SetColor(newColor)
			self:OnColorChanged(self.picker, self.picker, newColor )
			blockR = false
		end

		function colorHandler:SetRText(str)
			if blockR then return end
			self.boxR:SetText(str)
		end

		local blockG = false
		function colorHandler:ColorChangedG( wndHandler, wndControl, strText )
			local str = strText:sub(1,2) --ignore all other
			local num = tonumber(str, 16) --interpret as number in hex.
			if not num then return end --if the Text does not represent a number - cancel overwriting

			blockG = true
			local newColor = ApolloColor.new(self.color.r, num/255, self.color.b, self.color.a)
			self.picker:SetColor(newColor)
			self:OnColorChanged(self.picker, self.picker, newColor)
			blockG = false
		end

		function colorHandler:SetGText(str)
			if blockG then return end
			self.boxG:SetText(str)
		end

		local blockB = false
		function colorHandler:ColorChangedB( wndHandler, wndControl, strText )
			local str = strText:sub(1,2) --ignore all other
			local num = tonumber(str, 16) --interpret as number in hex.
			if not num then return end --if the Text does not represent a number - cancel overwriting

			blockB = true
			local newColor = ApolloColor.new(self.color.r, self.color.g, num/255, self.color.a)
			self.picker:SetColor(newColor)
			self:OnColorChanged(self.picker, self.picker, newColor)
			blockB = false
		end

		function colorHandler:SetBText(str)
			if blockB then return end
			self.boxB:SetText(str)
		end

		function colorHandler:OnNewColorClick(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
		end

		function colorHandler:OnOldColorClick(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
			local color = wndControl:GetBGColor()
			self:Pick(self.callback, color) --treat it as if we have opened the picker with this value.
		end

		function colorHandler:ClickCopy(...)
			self.copied = self.newColor:GetBGColor()
		end

		function colorHandler:ClickPaste(...)
			local color = self.copied
			self:Pick(self.callback, color) --treat it as if we have opened the picker with this value.
		end

		function colorHandler:Is2x(c)
			if c.r > 1 or c.g>1 or c.b>1 then
				return true, ApolloColor.new(c.r/2, c.g/2, c.b/2, c.a)
			else
				return false, c
			end
		end

		function colorHandler:Pick(callback, initial)
			if not initial then return end
			local color = ApolloColor.new(initial)

			self.dbl, color = self:Is2x(color)

			self.callback = callback
			self.color = color
			self.alpha = color.a

			self.oldColor:SetBGColor(ApolloColor.new(initial))
			self.picker:SetColor(color)
			self:OnColorChanged(self.picker, self.picker, color)

			colorPicker:Show(true)
		end

		if not self.mainFrame then
			self:InitMainFrame()
		end

		colorPicker = NLLC:LoadForm(FORM_COLORPICKER_TEMPLATE, self.mainFrame, colorHandler)
		colorHandler.header = colorPicker:FindChild("Header")
		colorHandler.picker = colorPicker:FindChild("Color")
		colorHandler.slider = colorPicker:FindChild("AlphaSlider")
		colorHandler.btn2x = colorPicker:FindChild("Checkbox_2x")
		colorHandler.boxR = colorPicker:FindChild("EditBoxR")
		colorHandler.boxG = colorPicker:FindChild("EditBoxG")
		colorHandler.boxB = colorPicker:FindChild("EditBoxB")
		colorHandler.boxA =	colorPicker:FindChild("EditBoxA")
		colorHandler.oldColor = colorPicker:FindChild("Color_old")
		colorHandler.newColor = colorPicker:FindChild("Color_new")

		colorHandler.header:SetText(L["Color Picker"])
		colorPicker:FindChild("Title_NewColor"):SetText(L["New Color:"])
		colorPicker:FindChild("Title_OldColor"):SetText(L["Old Color:"])
		colorPicker:FindChild("Button_Copy"):SetText(L["Copy"])
		colorPicker:FindChild("Button_Paste"):SetText(L["Paste"])

		colorHandler.btn2x:SetTooltip(L["tt2x"])

		function UIComponents:InitColorPicker() --replace the function, we do not want this to be done multiple times.
			return colorHandler
		end
		return colorHandler
	end

end

do--[[#####  Fontbox #####]] --:applyFontbox(parent, selector)
	local Fontbox = {}; Fontbox.__index = Fontbox
	local trans = setmetatable({
		["_I"] = "I",
		["_O"] = "O",
		["_BO"] = "BO",
		["_BBO"] = "BBO",
		["_BB"] = "BB",
		["_B"] = "B",
		["_Huge"] = "Huge",
		["_Small"] = "Small",
		["CRB_Dialog_Heading"] = "Dialog Heading",
	},{__index = function(t, k)
		if k:sub(1,4) == "CRB_" then
			t[k] = k:sub(5,-1)
		else
			t[k] = k
		end
		return t[k]
	end})

	local none = {"",[0] = 1}
	local none2 = {"", [""] = none, [0] = 1}
	local NoAndO = {"", "_O", [0] = 1}
	local NoAndI = {"", "_I", [0] = 1}
	local alot = {"", "_B", "_BB", "_BBO", "_BO", "_I", "_O", [0] = 1}

	local fonts = {
		[0] = 9, --0 is always the default
		"Courier", "CRB_Alien", "CRB_Button", "CRB_ButtonHeader", "CRB_Dialog",
		"CRB_Dialog_Heading", "CRB_Floater", "CRB_Header", "CRB_Interface", "CRB_Pixel",
		"CRB_ResourceOnly", "Default", "DefaultButton",	"Nameplates",
		Courier = none2,
		CRB_Alien = {
			[0] = 3,
			"Huge", "Large", "Medium", "Small",
			["Huge"] = none,
			["Large"] = none,
			["Medium"] = none,
			["Small"] = none,
		}, CRB_Button = none2,
		CRB_ButtonHeader = none2,
		CRB_Dialog = {
			"",
			[""] = NoAndI,
			[0] = 1,
		}, CRB_Dialog_Heading = {
			"", "_Huge", "_Small",
			[""] = none,
			["_Huge"] = none,
			["_Small"] = none,
			[0] = 1,
		}, CRB_Floater = {
			"Small", "Medium", "Large", "Huge", "Gigantic",
			Small = none,
			Medium = none,
			Large = none,
			Huge = NoAndO,
			Gigantic = NoAndO,
			[0] = 2,
		}, CRB_Header = {
			"Tiny", "Small", "Medium", "Large", "Huge", "Huger", "Gigantic",
			"9", "10", "11", "12", "13", "14", "16", "18", "20", "24",
			["Tiny"] = NoAndO, ["Small"] = NoAndO, ["Medium"] = NoAndO,
			["Large"] = NoAndO, ["Huge"] = NoAndO, ["Huger"] = NoAndO,
			["Gigantic"] = NoAndO, ["9"] = NoAndO, ["10"] = NoAndO, ["11"] = NoAndO,
			["12"] = NoAndO, ["13"] = NoAndO, ["14"] = {"", "_O", "NoDrop", [0] = 1},
			["16"] = NoAndO, ["18"] = NoAndO, ["20"] = NoAndO, ["24"] = NoAndO,
			[0] = 3,
		}, CRB_Interface = {
			"Tiny", "Small", "Medium", "Large", "7", "9", "10", "11", "12", "14", "16",
			["7"] = {"_BB",[0] = 1}, ["Tiny"] = {"_BB",[0] = 1},
			["Small"] = {"", "_B", "_BB", "_I", "_O", [0] = 1,},
			["Medium"] = alot, ["Large"] = alot, ["9"] = alot,
			["10"] = alot, ["11"] = alot, ["12"] = alot, ["14"] = alot,
			["16"] = alot,
			[0] = 3,
		}, CRB_Pixel = {"", [""] = NoAndO, [0] = 1},
		CRB_ResourceOnly = none2,
		Default = none2,
		DefaultButton = none2,
		Nameplates = none2,
		Subtitle = none2,
		Thick = none2,
	}
	local defA = fonts[0]
	local defB = fonts[fonts[defA]][0]
	local defC = fonts[fonts[defA]][fonts[fonts[defA]][defB]][0]

	local function getIndexes(x)
		--searches through the fonts -> returns first second and thrid index inside 'fonts'
		local y, z;

		for a, s1 in ipairs(fonts) do
			y = x:match("^"..s1.."(.*)$")
			if y then
				for b, s2 in ipairs(fonts[s1]) do
					z = y:match("^"..s2.."(.*)$")
					if z then
						for c, s in ipairs(fonts[s1][s2]) do
							if z == tostring(s) then
								return a, b, c
							end
						end
					end
				end
			end
		end

		return nil
	end

	function Fontbox:SetIndex(a, b, c)
		--make sure a,b,c are given.
		if a and not b then
			local aIdx = fonts[a]
			b = fonts[aIdx][0]
			local bIdx = fonts[aIdx][b]
			c = fonts[aIdx][bIdx][0]
		elseif b and not c then
			a = a or self.a or fonts[0]
			local aIdx = fonts[a]
			local bIdx = fonts[aIdx][b]
			c = fonts[aIdx][bIdx][0]
		elseif c then
			a = a or self.a or fonts[0]
			b = b or self.b or fonts[fonts[a]][0]
		elseif not a and not b and not c then
			a = fonts[0] --9
			local aIdx = fonts[a] --'CRB_Interface'
			b = fonts[aIdx][0] --3
			local bIdx = fonts[aIdx][b]
			c = fonts[aIdx][bIdx][0]
		end

		local strA = fonts[a]
		local tblA = fonts[strA]
		local strB = tblA[b]
		local tblB = tblA[strB]
		local strC = tblB[c]

		self.txtName:SetText(trans[strA])
		self.txtSize:SetText(trans[strB])
		self.txtAttr:SetText(trans[strC])

		self.a = a
		self.b = b
		self.c = c

		self.btnLSize:Enable(#tblA > 1)
		self.btnRSize:Enable(#tblA > 1)

		self.btnLAttr:Enable(#tblB > 1)
		self.btnRAttr:Enable(#tblB > 1)

		if not self.block then
			self.block = true
			self.opt:Set(strA..strB..strC)
			self.block = false
		end
	end

	function Fontbox:OnUpdate(val)
		if self.block then return end
		self.block = true
		local a,b,c;
		if val and type(val) == "string" then
			a, b, c = getIndexes(val)
		end
		if not a then --return to default!
			a, b, c = defA, defB, defC
		end

		self:SetIndex(a,b,c)

		self.block = false
	end

	function Fontbox:NameLeft(wndControl, wndHandler)
		if wndControl ~= wndHandler then return end
		local a = self.a-1
		if a == 0 then
			a = #fonts
		end
		self:SetIndex(a, nil, nil)
	end
	function Fontbox:NameRight(wndControl, wndHandler)
		if wndControl ~= wndHandler then return end
		local a = self.a+1
		if a > #fonts then
			a = 1
		end
		self:SetIndex(a, nil, nil)
	end
	function Fontbox:SizeLeft(wndControl, wndHandler)
		if wndControl ~= wndHandler then return end
		local b = self.b-1
		if b == 0 then
			local tbl = fonts[fonts[self.a]]
			b = #tbl
		end
		self:SetIndex(nil, b, nil)
	end
	function Fontbox:SizeRight(wndControl, wndHandler)
		if wndControl ~= wndHandler then return end
		local b = self.b+1
		local tbl = fonts[fonts[self.a]]
		if b > #tbl then
			b = 1
		end
		self:SetIndex(nil, b, nil)
	end
	function Fontbox:AttributeLeft(wndControl, wndHandler)
		if wndControl ~= wndHandler then return end
		local c = self.c-1
		if c == 0 then
			local tbl = fonts[fonts[self.a]][fonts[fonts[self.a]][self.b]]
			c = #tbl
		end
		self:SetIndex(nil, nil, c)
	end
	function Fontbox:AttributeRight(wndControl, wndHandler)
		if wndControl ~= wndHandler then return end
		local c = self.c+1
		local tbl = fonts[fonts[self.a]][fonts[fonts[self.a]][self.b]]
		if c > #tbl then
			c = 1
		end
		self:SetIndex(nil, nil, c)
	end

	function UIComponents:applyFontbox(parent, selector)
		local handler = setmetatable({
			opt = selector,
			block = false,
			a = defA,
			b = defB,
			c = defC,
		}, Fontbox)
		local form = NLLC:LoadForm(FORM_FONTBOX_TEMPLATE, parent, handler)
		handler.txtName = form:FindChild("BaseName:Text")
		handler.txtSize = form:FindChild("Size:Text")
		handler.txtAttr = form:FindChild("Attribute:Text")
		handler.btnLSize = form:FindChild("Size:Left")
		handler.btnRSize = form:FindChild("Size:Right")
		handler.btnLAttr = form:FindChild("Attribute:Left")
		handler.btnRAttr = form:FindChild("Attribute:Right")

		selector:OnUpdate(handler, "OnUpdate")
		return handler
	end
end

do--[[#####  Spritebox #####]] --:applySpritebox(parent, selector)
	local Spritebox = {}; Spritebox.__index = Spritebox

	function Spritebox:OpenSpriteList(...)
		self:InitSpriteList()
	end

	function UIComponents:applySpritebox(parent, selector)
		local handler = setmetatable({
			opt = selector,
		}, Spritebox)
		local form = NLLC:LoadForm(FORM_SPRITEBOX_TEMPLATE, parent, handler)
		return self:applyTextbox(form, selector)
	end

	function Spritebox:InitSpriteList()
		local tmp = {
			"",
			"CRB_TargetFrameSprites:sprTF_PathSoldier",
			"BasicSprites:WhiteFill",
			"BasicSprites:WhiteCircle",
			"IconSprites:Icon_Windows_UI_CRB_Checkmark",
			"ClientSprites:LootCloseBox_Holo",
			"CRB_InterfaceMenuList:spr_InterfaceMenuList_BlueFlagStretch",
			"CRB_InterfaceMenuList:spr_InterfaceMenuList_RedFlagStretch",
			"CRB_InterfaceMenuList:spr_InterfaceMenuList_SilverFlagStretch",
			"CRB_Nameplates:sprNP_HighLevel",
			"IconSprites:Icon_RuneCrafting_Runecrafting_Water_Colored",
			"IconSprites:Icon_RuneCrafting_Runecrafting_Logic_Colored",
			"IconSprites:Icon_RuneCrafting_Runecrafting_Life_Colored",
			"IconSprites:Icon_RuneCrafting_Runecrafting_Fusion_Colored",
			"IconSprites:Icon_RuneCrafting_Runecrafting_Fire_Colored",
			"IconSprites:Icon_RuneCrafting_Runecrafting_Earth_Colored",
			"IconSprites:Icon_RuneCrafting_Runecrafting_Air_Colored",
			"IconSprites:Icon_Windows_UI_CRB_Marker_Crosshair",
			"IconSprites:Icon_Windows_UI_CRB_Marker_Bomb",
			"IconSprites:Icon_Windows_UI_CRB_Marker_Chicken",
			"IconSprites:Icon_Windows_UI_CRB_Marker_Ghost",
			"IconSprites:Icon_Windows_UI_CRB_Marker_Mask",
			"IconSprites:Icon_Windows_UI_CRB_Marker_Octopus",
			"IconSprites:Icon_Windows_UI_CRB_Marker_Pig",
			"IconSprites:Icon_Windows_UI_CRB_Marker_Toaster",
			"IconSprites:Icon_Windows_UI_CRB_Marker_UFO",
			"IconSprites:Icon_Windows_UI_CRB_Tooltip_Restricted",
		}

		local handler = {}
		local form = NLLC:LoadForm(FORM_SPRITELIST_TEMPLATE, UIComponents.mainFrame, handler)
		local list = form:FindChild("List")

		local pixie = {
			strSprite = "",
			loc = {
				fPoints = {0, 0, 1, 1},
				nOffsets = {3, 3,-3,-3},
			},
		}

		local left, right;
		for i = 1, #tmp+(#tmp%2), 1 do
			if not tmp[i] then --can only happen to the right one!
				right:Show(false, true)
			elseif i%2 == 1 then --left (create new line)
				local line = NLLC:LoadForm(FORM_SPRITELINE_TEMPLATE, list, handler)
				left, right = line:FindChild("Left"), line:FindChild("Right")

				pixie.strSprite = tmp[i]
				left:AddPixie(pixie)
				left:SetData(tmp[i])
			else --right
				pixie.strSprite = tmp[i]
				right:AddPixie(pixie)
				right:SetData(tmp[i])
			end
		end

		list:ArrangeChildrenVert()
		list:RecalculateContentExtents()


		function handler:SpriteSelected(wndControl, wndHandler)
			if wndControl ~= wndHandler then return end
			self.opt:Set(wndControl:GetData())
			form:Show(false, true)
		end

		--dont change to self! they are different tables!
		function Spritebox:InitSpriteList()
			handler.opt = self.opt
			form:Show(true, true)
		end

		self:InitSpriteList()
	end
end
